"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cliMachine = void 0;
const xstate_1 = require("xstate");
const services_1 = require("./services");
exports.cliMachine = (0, xstate_1.createMachine)({
    id: 'CLI',
    initial: 'idle',
    context: {
        name: '',
        template: '',
        path: '',
        error: null,
        currentStepMessage: null,
        completedStepMessages: []
    },
    states: {
        idle: {
            on: {
                START: 'executing',
                SHOW_HELP: 'help',
                SHOW_VERSION: 'version'
            }
        },
        help: {
            type: 'final'
        },
        version: {
            type: 'final'
        },
        executing: {
            initial: 'validateProjectName',
            states: {
                validateProjectName: {
                    entry: (0, xstate_1.assign)({
                        currentStepMessage: (context) => `Checking the project name: "${context.name}"`
                    }),
                    invoke: {
                        src: 'validateProjectName',
                        onDone: {
                            actions: (0, xstate_1.assign)({
                                completedStepMessages: (context) => [
                                    ...context.completedStepMessages,
                                    `Project name: "${context.name}" is valid`
                                ]
                            }),
                            target: 'validateTemplate'
                        },
                        onError: {
                            actions: (0, xstate_1.assign)({
                                error: (_, event) => event.data
                            }),
                            target: 'failed'
                        }
                    }
                },
                validateTemplate: {
                    entry: (0, xstate_1.assign)({
                        currentStepMessage: (context) => `Checking the template: "${context.template}"`
                    }),
                    invoke: {
                        src: 'validateTemplate',
                        onDone: {
                            actions: (0, xstate_1.assign)({
                                completedStepMessages: (context) => [
                                    ...context.completedStepMessages,
                                    `Template: "${context.template}" is valid`
                                ]
                            }),
                            target: 'validateProjectPath'
                        },
                        onError: {
                            actions: (0, xstate_1.assign)({
                                error: (_, event) => event.data
                            }),
                            target: 'failed'
                        }
                    }
                },
                validateProjectPath: {
                    entry: (0, xstate_1.assign)({
                        currentStepMessage: (context) => `Checking the project path: "${context.path}"`
                    }),
                    invoke: {
                        src: 'validateProjectPath',
                        onDone: {
                            actions: (0, xstate_1.assign)({
                                completedStepMessages: (context) => [
                                    ...context.completedStepMessages,
                                    `Project path: "${context.path}" is valid`
                                ]
                            }),
                            target: 'bootstrapProject'
                        },
                        onError: {
                            actions: (0, xstate_1.assign)({
                                error: (_, event) => event.data
                            }),
                            target: 'failed'
                        }
                    }
                },
                bootstrapProject: {
                    entry: (0, xstate_1.assign)({
                        currentStepMessage: (context) => `Creating a new project in "${context.path}"`
                    }),
                    invoke: {
                        src: 'bootstrapProject',
                        onDone: {
                            actions: (0, xstate_1.assign)({
                                completedStepMessages: (context) => [
                                    ...context.completedStepMessages,
                                    `The project was initialized in "${context.path}"`
                                ]
                            }),
                            target: 'generatePackageJSONfile'
                        },
                        onError: {
                            actions: (0, xstate_1.assign)({
                                error: (_, event) => event.data
                            }),
                            target: 'failed'
                        }
                    }
                },
                generatePackageJSONfile: {
                    entry: (0, xstate_1.assign)({
                        currentStepMessage: (context) => `Generating the package.json for "${context.template}" template`
                    }),
                    invoke: {
                        src: 'generatePackageJSONfile',
                        onDone: {
                            actions: (0, xstate_1.assign)({
                                completedStepMessages: (context) => [
                                    ...context.completedStepMessages,
                                    `The package.json for "${context.template}" template was generated`
                                ]
                            }),
                            target: 'copyTemplateFiles'
                        },
                        onError: {
                            actions: (0, xstate_1.assign)({
                                error: (_, event) => event.data
                            }),
                            target: 'failed'
                        }
                    }
                },
                copyTemplateFiles: {
                    entry: (0, xstate_1.assign)({
                        currentStepMessage: (context) => `Copying files from template "${context.template}" to "${context.path}"`
                    }),
                    invoke: {
                        src: 'copyTemplateFiles',
                        onDone: {
                            actions: (0, xstate_1.assign)({
                                completedStepMessages: (context) => [
                                    ...context.completedStepMessages,
                                    `The files from "${context.template}" template were copied to "${context.path}"`
                                ]
                            }),
                            target: 'installPackageDependencies'
                        },
                        onError: {
                            actions: (0, xstate_1.assign)({
                                error: (_, event) => event.data
                            }),
                            target: 'failed'
                        }
                    }
                },
                installPackageDependencies: {
                    entry: (0, xstate_1.assign)({
                        currentStepMessage: (context) => `Installing packages for "${context.template}" template`
                    }),
                    invoke: {
                        src: 'installPackageDependencies',
                        onDone: {
                            actions: (0, xstate_1.assign)({
                                completedStepMessages: (context) => [
                                    ...context.completedStepMessages,
                                    `The "${context.template}" template packages were installed`
                                ]
                            }),
                            target: 'initializeGitRepository'
                        },
                        onError: {
                            actions: (0, xstate_1.assign)({
                                error: (_, event) => event.data
                            }),
                            target: 'failed'
                        }
                    }
                },
                initializeGitRepository: {
                    entry: (0, xstate_1.assign)({
                        currentStepMessage: (context) => `Initializing a git repository in "${context.path}"`
                    }),
                    invoke: {
                        src: 'initializeGitRepository',
                        onDone: {
                            actions: (0, xstate_1.assign)({
                                completedStepMessages: (context) => [
                                    ...context.completedStepMessages,
                                    `The git repository in "${context.path}" was initialized`
                                ]
                            }),
                            target: 'completed'
                        },
                        onError: {
                            actions: (0, xstate_1.assign)({
                                error: (_, event) => event.data
                            }),
                            target: 'failed'
                        }
                    }
                },
                completed: {
                    entry: (0, xstate_1.assign)({
                        currentStepMessage: null
                    }),
                    type: 'final'
                },
                failed: { type: 'final' }
            },
            onDone: [{ target: 'error', cond: 'hasError' }, { target: 'success' }]
        },
        error: {
            type: 'final'
        },
        success: {
            type: 'final'
        }
    }
}, {
    guards: {
        hasError: ({ error }) => error !== null
    },
    services: {
        validateProjectName: ({ name }) => (0, services_1.validateProjectName)(name),
        validateTemplate: ({ template }) => (0, services_1.validateTemplate)(template),
        validateProjectPath: ({ path }) => (0, services_1.validateProjectPath)(path),
        bootstrapProject: ({ path }) => (0, services_1.bootstrapProject)(path),
        generatePackageJSONfile: ({ name, template, path }) => (0, services_1.generatePackageJSONfile)(name, template, path),
        copyTemplateFiles: ({ template, path }) => (0, services_1.copyTemplateFiles)(template, path),
        installPackageDependencies: ({ template, path }) => (0, services_1.installPackageDependencies)(template, path),
        initializeGitRepository: ({ path }) => (0, services_1.initializeGitRepository)(path)
    }
});
