"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CLI = void 0;
const ink_1 = require("ink");
const ink_spinner_1 = __importDefault(require("ink-spinner"));
const react_1 = __importStar(require("react"));
const react_2 = require("@xstate/react");
const cli_machine_1 = require("./cli-machine");
const highlighted_text_1 = require("./components/highlighted-text");
const page_layout_1 = require("./components/page-layout");
const utils_1 = require("./utils");
const CLI = ({ pkgName, cliVersion, description, help, version, name, template, appPath }) => {
    var _a, _b;
    const [state, send] = (0, react_2.useMachine)(cli_machine_1.cliMachine, {
        context: {
            name,
            template,
            path: appPath
        }
    });
    (0, react_1.useEffect)(() => {
        if (help) {
            send('SHOW_HELP');
        }
        else if (version) {
            send('SHOW_VERSION');
        }
        else {
            send('START');
        }
    }, [help, version, send]);
    if (state.matches('version')) {
        return react_1.default.createElement(ink_1.Text, null, cliVersion);
    }
    else if (state.matches('help')) {
        return (react_1.default.createElement(page_layout_1.PageLayout, { version: cliVersion, name: pkgName, description: description },
            react_1.default.createElement(highlighted_text_1.HighlightedText, { text: `Usage: ${pkgName} [options] "<project-name>"`, color: "green" }),
            react_1.default.createElement(ink_1.Box, { flexDirection: "column", marginTop: 1 },
                react_1.default.createElement(ink_1.Box, null, "Options:"),
                react_1.default.createElement(ink_1.Box, { flexDirection: "column", marginLeft: 1 },
                    react_1.default.createElement(ink_1.Box, null,
                        react_1.default.createElement(ink_1.Box, { width: 32 }, "-h,--help"),
                        react_1.default.createElement(ink_1.Text, null, "Show Help")),
                    react_1.default.createElement(ink_1.Box, null,
                        react_1.default.createElement(ink_1.Box, { width: 32 }, "-v,--version"),
                        react_1.default.createElement(ink_1.Text, null, "Show CLI Version")),
                    react_1.default.createElement(ink_1.Box, null,
                        react_1.default.createElement(ink_1.Box, { width: 32 }, `-t,--template <template-name>`),
                        react_1.default.createElement(ink_1.Text, null, "Specify the project template (default: node)"))))));
    }
    else if (state.matches('executing')) {
        return (react_1.default.createElement(page_layout_1.PageLayout, { version: cliVersion, name: pkgName, description: description },
            state.context.completedStepMessages.map((message, index) => (react_1.default.createElement(ink_1.Box, { key: index, paddingLeft: 3 },
                react_1.default.createElement(ink_1.Box, { paddingRight: 3 },
                    react_1.default.createElement(ink_1.Text, null, (0, utils_1.logSymbols)('success'))),
                react_1.default.createElement(highlighted_text_1.HighlightedText, { text: message, color: "green" })))),
            state.context.currentStepMessage ? (react_1.default.createElement(ink_1.Box, { paddingLeft: 3 },
                react_1.default.createElement(ink_spinner_1.default, { type: "clock" }),
                react_1.default.createElement(highlighted_text_1.HighlightedText, { text: state.context.currentStepMessage, color: "yellow" }))) : null));
    }
    else if (state.matches('success')) {
        return (react_1.default.createElement(page_layout_1.PageLayout, { version: cliVersion, name: pkgName, description: description },
            react_1.default.createElement(highlighted_text_1.HighlightedText, { text: `The project "${name}" has been successfully created in "${appPath}"`, color: "green" })));
    }
    else if (state.matches('error')) {
        return (react_1.default.createElement(page_layout_1.PageLayout, { version: cliVersion, name: pkgName, description: description },
            react_1.default.createElement(highlighted_text_1.HighlightedText, { text: `Failed to create the project "${name}" in "${appPath}"`, color: "red" }),
            react_1.default.createElement(ink_1.Box, { paddingLeft: 3 },
                react_1.default.createElement(ink_1.Box, { paddingRight: 3 },
                    react_1.default.createElement(ink_1.Text, null, (0, utils_1.logSymbols)('error'))),
                react_1.default.createElement(highlighted_text_1.HighlightedText, { text: ((_b = (_a = state.context) === null || _a === void 0 ? void 0 : _a.error) === null || _b === void 0 ? void 0 : _b.message) ? state.context.error.message : '', color: "red" }))));
    }
    else {
        return null;
    }
};
exports.CLI = CLI;
