"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseProgramArgs = exports.getPkgJSON = exports.capitalize = exports.sleep = exports.logSymbols = exports.isUnicodeSupported = exports.getBigText = void 0;
const chalk_1 = __importDefault(require("chalk"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const minimist_1 = __importDefault(require("minimist"));
const cfonts_1 = __importDefault(require("cfonts"));
const constants_1 = require("./constants");
function getBigText(text) {
    const output = cfonts_1.default.render(text, {
        font: 'block',
        align: 'left',
        colors: ['system'],
        background: 'transparent',
        letterSpacing: 1,
        lineHeight: 1,
        space: true,
        maxLength: '0' // define how many character can be on one line
    });
    return output.string;
}
exports.getBigText = getBigText;
function isUnicodeSupported() {
    if (process.platform !== 'win32') {
        return process.env.TERM !== 'linux'; // Linux console (kernel)
    }
    return (Boolean(process.env.CI) ||
        Boolean(process.env.WT_SESSION) || // Windows Terminal
        process.env.ConEmuTask === '{cmd::Cmder}' || // ConEmu and cmder
        process.env.TERM_PROGRAM === 'vscode' ||
        process.env.TERM === 'xterm-256color' ||
        process.env.TERM === 'alacritty');
}
exports.isUnicodeSupported = isUnicodeSupported;
function logSymbols(symbol) {
    const main = {
        info: chalk_1.default.blue('ℹ'),
        success: chalk_1.default.green('✔'),
        warning: chalk_1.default.yellow('⚠'),
        error: chalk_1.default.red('✖')
    };
    const fallback = {
        info: chalk_1.default.blue('i'),
        success: chalk_1.default.green('√'),
        warning: chalk_1.default.yellow('‼'),
        error: chalk_1.default.red('×')
    };
    return isUnicodeSupported() ? main[symbol] : fallback[symbol];
}
exports.logSymbols = logSymbols;
function sleep(ms) {
    return new Promise((resolve) => setTimeout(resolve, ms));
}
exports.sleep = sleep;
function capitalize(message) {
    if (typeof message !== 'string')
        return '';
    return message.charAt(0).toUpperCase() + message.slice(1);
}
exports.capitalize = capitalize;
const getPkgJSON = () => {
    return JSON.parse(fs_extra_1.default.readFileSync(constants_1.PKG_JSON_PATH, 'utf8'));
};
exports.getPkgJSON = getPkgJSON;
const parseProgramArgs = (args) => {
    const parsedArgs = (0, minimist_1.default)(args, {
        string: ['template'],
        boolean: ['version', 'help'],
        alias: { h: 'help', v: 'version', t: 'template' },
        default: { template: 'node' }
    });
    const help = parsedArgs._.length > 1 || Object.keys(parsedArgs).length > 7 ? true : parsedArgs.help;
    const name = parsedArgs._.length > 0 ? parsedArgs._[0] : '';
    const version = parsedArgs.version;
    const template = parsedArgs.template;
    return {
        help,
        name,
        version,
        template
    };
};
exports.parseProgramArgs = parseProgramArgs;
